Method: PDF::Reader::Stream#initialize

Defined in:
lib/pdf/reader/stream.rb

#initialize(hash, data) ⇒ Stream

Creates a new stream with the specified dictionary and data. The dictionary should be a standard ruby hash, the data should be a standard ruby string.



41
42
43
44
45
# File 'lib/pdf/reader/stream.rb', line 41

def initialize(hash, data)
  @hash = hash
  @data = data
  @udata = nil
end